/* da-vfs-gio.h generated by valac 0.10.4, the Vala compiler, do not modify */


#ifndef __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_VFS_GIO_H__
#define __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_VFS_GIO_H__

#include <glib.h>
#include <glib-object.h>
#include <libdesktop-agnostic/vfs.h>

G_BEGIN_DECLS


#define DESKTOP_AGNOSTIC_VFS_TYPE_GIO_IMPLEMENTATION (desktop_agnostic_vfs_gio_implementation_get_type ())
#define DESKTOP_AGNOSTIC_VFS_GIO_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_GIO_IMPLEMENTATION, DesktopAgnosticVFSGIOImplementation))
#define DESKTOP_AGNOSTIC_VFS_GIO_IMPLEMENTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_GIO_IMPLEMENTATION, DesktopAgnosticVFSGIOImplementationClass))
#define DESKTOP_AGNOSTIC_VFS_IS_GIO_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_GIO_IMPLEMENTATION))
#define DESKTOP_AGNOSTIC_VFS_IS_GIO_IMPLEMENTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_GIO_IMPLEMENTATION))
#define DESKTOP_AGNOSTIC_VFS_GIO_IMPLEMENTATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_GIO_IMPLEMENTATION, DesktopAgnosticVFSGIOImplementationClass))

typedef struct _DesktopAgnosticVFSGIOImplementation DesktopAgnosticVFSGIOImplementation;
typedef struct _DesktopAgnosticVFSGIOImplementationClass DesktopAgnosticVFSGIOImplementationClass;
typedef struct _DesktopAgnosticVFSGIOImplementationPrivate DesktopAgnosticVFSGIOImplementationPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GIO (desktop_agnostic_vfs_file_gio_get_type ())
#define DESKTOP_AGNOSTIC_VFS_FILE_GIO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GIO, DesktopAgnosticVFSFileGIO))
#define DESKTOP_AGNOSTIC_VFS_FILE_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GIO, DesktopAgnosticVFSFileGIOClass))
#define DESKTOP_AGNOSTIC_VFS_IS_FILE_GIO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GIO))
#define DESKTOP_AGNOSTIC_VFS_IS_FILE_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GIO))
#define DESKTOP_AGNOSTIC_VFS_FILE_GIO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GIO, DesktopAgnosticVFSFileGIOClass))

typedef struct _DesktopAgnosticVFSFileGIO DesktopAgnosticVFSFileGIO;
typedef struct _DesktopAgnosticVFSFileGIOClass DesktopAgnosticVFSFileGIOClass;
typedef struct _DesktopAgnosticVFSFileGIOPrivate DesktopAgnosticVFSFileGIOPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GIO (desktop_agnostic_vfs_file_monitor_gio_get_type ())
#define DESKTOP_AGNOSTIC_VFS_FILE_MONITOR_GIO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GIO, DesktopAgnosticVFSFileMonitorGIO))
#define DESKTOP_AGNOSTIC_VFS_FILE_MONITOR_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GIO, DesktopAgnosticVFSFileMonitorGIOClass))
#define DESKTOP_AGNOSTIC_VFS_IS_FILE_MONITOR_GIO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GIO))
#define DESKTOP_AGNOSTIC_VFS_IS_FILE_MONITOR_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GIO))
#define DESKTOP_AGNOSTIC_VFS_FILE_MONITOR_GIO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GIO, DesktopAgnosticVFSFileMonitorGIOClass))

typedef struct _DesktopAgnosticVFSFileMonitorGIO DesktopAgnosticVFSFileMonitorGIO;
typedef struct _DesktopAgnosticVFSFileMonitorGIOClass DesktopAgnosticVFSFileMonitorGIOClass;
typedef struct _DesktopAgnosticVFSFileMonitorGIOPrivate DesktopAgnosticVFSFileMonitorGIOPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GIO (desktop_agnostic_vfs_trash_gio_get_type ())
#define DESKTOP_AGNOSTIC_VFS_TRASH_GIO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GIO, DesktopAgnosticVFSTrashGIO))
#define DESKTOP_AGNOSTIC_VFS_TRASH_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GIO, DesktopAgnosticVFSTrashGIOClass))
#define DESKTOP_AGNOSTIC_VFS_IS_TRASH_GIO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GIO))
#define DESKTOP_AGNOSTIC_VFS_IS_TRASH_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GIO))
#define DESKTOP_AGNOSTIC_VFS_TRASH_GIO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GIO, DesktopAgnosticVFSTrashGIOClass))

typedef struct _DesktopAgnosticVFSTrashGIO DesktopAgnosticVFSTrashGIO;
typedef struct _DesktopAgnosticVFSTrashGIOClass DesktopAgnosticVFSTrashGIOClass;
typedef struct _DesktopAgnosticVFSTrashGIOPrivate DesktopAgnosticVFSTrashGIOPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GIO (desktop_agnostic_vfs_volume_gio_get_type ())
#define DESKTOP_AGNOSTIC_VFS_VOLUME_GIO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GIO, DesktopAgnosticVFSVolumeGIO))
#define DESKTOP_AGNOSTIC_VFS_VOLUME_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GIO, DesktopAgnosticVFSVolumeGIOClass))
#define DESKTOP_AGNOSTIC_VFS_IS_VOLUME_GIO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GIO))
#define DESKTOP_AGNOSTIC_VFS_IS_VOLUME_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GIO))
#define DESKTOP_AGNOSTIC_VFS_VOLUME_GIO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GIO, DesktopAgnosticVFSVolumeGIOClass))

typedef struct _DesktopAgnosticVFSVolumeGIO DesktopAgnosticVFSVolumeGIO;
typedef struct _DesktopAgnosticVFSVolumeGIOClass DesktopAgnosticVFSVolumeGIOClass;
typedef struct _DesktopAgnosticVFSVolumeGIOPrivate DesktopAgnosticVFSVolumeGIOPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GIO (desktop_agnostic_vfs_volume_monitor_gio_get_type ())
#define DESKTOP_AGNOSTIC_VFS_VOLUME_MONITOR_GIO(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GIO, DesktopAgnosticVFSVolumeMonitorGIO))
#define DESKTOP_AGNOSTIC_VFS_VOLUME_MONITOR_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GIO, DesktopAgnosticVFSVolumeMonitorGIOClass))
#define DESKTOP_AGNOSTIC_VFS_IS_VOLUME_MONITOR_GIO(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GIO))
#define DESKTOP_AGNOSTIC_VFS_IS_VOLUME_MONITOR_GIO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GIO))
#define DESKTOP_AGNOSTIC_VFS_VOLUME_MONITOR_GIO_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GIO, DesktopAgnosticVFSVolumeMonitorGIOClass))

typedef struct _DesktopAgnosticVFSVolumeMonitorGIO DesktopAgnosticVFSVolumeMonitorGIO;
typedef struct _DesktopAgnosticVFSVolumeMonitorGIOClass DesktopAgnosticVFSVolumeMonitorGIOClass;
typedef struct _DesktopAgnosticVFSVolumeMonitorGIOPrivate DesktopAgnosticVFSVolumeMonitorGIOPrivate;

struct _DesktopAgnosticVFSGIOImplementation {
	GObject parent_instance;
	DesktopAgnosticVFSGIOImplementationPrivate * priv;
};

struct _DesktopAgnosticVFSGIOImplementationClass {
	GObjectClass parent_class;
};

struct _DesktopAgnosticVFSFileGIO {
	DesktopAgnosticVFSFile parent_instance;
	DesktopAgnosticVFSFileGIOPrivate * priv;
};

struct _DesktopAgnosticVFSFileGIOClass {
	DesktopAgnosticVFSFileClass parent_class;
};

struct _DesktopAgnosticVFSFileMonitorGIO {
	GObject parent_instance;
	DesktopAgnosticVFSFileMonitorGIOPrivate * priv;
};

struct _DesktopAgnosticVFSFileMonitorGIOClass {
	GObjectClass parent_class;
};

struct _DesktopAgnosticVFSTrashGIO {
	GObject parent_instance;
	DesktopAgnosticVFSTrashGIOPrivate * priv;
};

struct _DesktopAgnosticVFSTrashGIOClass {
	GObjectClass parent_class;
};

struct _DesktopAgnosticVFSVolumeGIO {
	GObject parent_instance;
	DesktopAgnosticVFSVolumeGIOPrivate * priv;
};

struct _DesktopAgnosticVFSVolumeGIOClass {
	GObjectClass parent_class;
};

struct _DesktopAgnosticVFSVolumeMonitorGIO {
	GObject parent_instance;
	DesktopAgnosticVFSVolumeMonitorGIOPrivate * priv;
};

struct _DesktopAgnosticVFSVolumeMonitorGIOClass {
	GObjectClass parent_class;
};


GType desktop_agnostic_vfs_gio_implementation_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSGIOImplementation* desktop_agnostic_vfs_gio_implementation_new (void);
DesktopAgnosticVFSGIOImplementation* desktop_agnostic_vfs_gio_implementation_construct (GType object_type);
GType register_plugin (void);
GType desktop_agnostic_vfs_file_gio_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSFileGIO* desktop_agnostic_vfs_file_gio_new (void);
DesktopAgnosticVFSFileGIO* desktop_agnostic_vfs_file_gio_construct (GType object_type);
GType desktop_agnostic_vfs_file_monitor_gio_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSFileMonitorGIO* desktop_agnostic_vfs_file_monitor_gio_new (DesktopAgnosticVFSFileGIO* file);
DesktopAgnosticVFSFileMonitorGIO* desktop_agnostic_vfs_file_monitor_gio_construct (GType object_type, DesktopAgnosticVFSFileGIO* file);
GType desktop_agnostic_vfs_trash_gio_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSTrashGIO* desktop_agnostic_vfs_trash_gio_new (void);
DesktopAgnosticVFSTrashGIO* desktop_agnostic_vfs_trash_gio_construct (GType object_type);
GType desktop_agnostic_vfs_volume_gio_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSVolumeGIO* desktop_agnostic_vfs_volume_gio_new (void);
DesktopAgnosticVFSVolumeGIO* desktop_agnostic_vfs_volume_gio_construct (GType object_type);
GType desktop_agnostic_vfs_volume_monitor_gio_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSVolumeMonitorGIO* desktop_agnostic_vfs_volume_monitor_gio_new (void);
DesktopAgnosticVFSVolumeMonitorGIO* desktop_agnostic_vfs_volume_monitor_gio_construct (GType object_type);


G_END_DECLS

#endif
