/* da-vfs-gnome-vfs.h generated by valac 0.10.4, the Vala compiler, do not modify */


#ifndef __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_VFS_GNOME_VFS_H__
#define __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_VFS_GNOME_VFS_H__

#include <glib.h>
#include <glib-object.h>
#include <libdesktop-agnostic/vfs.h>
#include <libgnomevfs/gnome-vfs.h>

G_BEGIN_DECLS


#define DESKTOP_AGNOSTIC_VFS_TYPE_GNOME_VFS_IMPLEMENTATION (desktop_agnostic_vfs_gnome_vfs_implementation_get_type ())
#define DESKTOP_AGNOSTIC_VFS_GNOME_VFS_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_GNOME_VFS_IMPLEMENTATION, DesktopAgnosticVFSGnomeVFSImplementation))
#define DESKTOP_AGNOSTIC_VFS_GNOME_VFS_IMPLEMENTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_GNOME_VFS_IMPLEMENTATION, DesktopAgnosticVFSGnomeVFSImplementationClass))
#define DESKTOP_AGNOSTIC_VFS_IS_GNOME_VFS_IMPLEMENTATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_GNOME_VFS_IMPLEMENTATION))
#define DESKTOP_AGNOSTIC_VFS_IS_GNOME_VFS_IMPLEMENTATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_GNOME_VFS_IMPLEMENTATION))
#define DESKTOP_AGNOSTIC_VFS_GNOME_VFS_IMPLEMENTATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_GNOME_VFS_IMPLEMENTATION, DesktopAgnosticVFSGnomeVFSImplementationClass))

typedef struct _DesktopAgnosticVFSGnomeVFSImplementation DesktopAgnosticVFSGnomeVFSImplementation;
typedef struct _DesktopAgnosticVFSGnomeVFSImplementationClass DesktopAgnosticVFSGnomeVFSImplementationClass;
typedef struct _DesktopAgnosticVFSGnomeVFSImplementationPrivate DesktopAgnosticVFSGnomeVFSImplementationPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GNOME_VFS (desktop_agnostic_vfs_file_gnome_vfs_get_type ())
#define DESKTOP_AGNOSTIC_VFS_FILE_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GNOME_VFS, DesktopAgnosticVFSFileGnomeVFS))
#define DESKTOP_AGNOSTIC_VFS_FILE_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GNOME_VFS, DesktopAgnosticVFSFileGnomeVFSClass))
#define DESKTOP_AGNOSTIC_VFS_IS_FILE_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_IS_FILE_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_FILE_GNOME_VFS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_GNOME_VFS, DesktopAgnosticVFSFileGnomeVFSClass))

typedef struct _DesktopAgnosticVFSFileGnomeVFS DesktopAgnosticVFSFileGnomeVFS;
typedef struct _DesktopAgnosticVFSFileGnomeVFSClass DesktopAgnosticVFSFileGnomeVFSClass;
typedef struct _DesktopAgnosticVFSFileGnomeVFSPrivate DesktopAgnosticVFSFileGnomeVFSPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GNOME_VFS (desktop_agnostic_vfs_file_monitor_gnome_vfs_get_type ())
#define DESKTOP_AGNOSTIC_VFS_FILE_MONITOR_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GNOME_VFS, DesktopAgnosticVFSFileMonitorGnomeVFS))
#define DESKTOP_AGNOSTIC_VFS_FILE_MONITOR_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GNOME_VFS, DesktopAgnosticVFSFileMonitorGnomeVFSClass))
#define DESKTOP_AGNOSTIC_VFS_IS_FILE_MONITOR_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_IS_FILE_MONITOR_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_FILE_MONITOR_GNOME_VFS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_FILE_MONITOR_GNOME_VFS, DesktopAgnosticVFSFileMonitorGnomeVFSClass))

typedef struct _DesktopAgnosticVFSFileMonitorGnomeVFS DesktopAgnosticVFSFileMonitorGnomeVFS;
typedef struct _DesktopAgnosticVFSFileMonitorGnomeVFSClass DesktopAgnosticVFSFileMonitorGnomeVFSClass;
typedef struct _DesktopAgnosticVFSFileMonitorGnomeVFSPrivate DesktopAgnosticVFSFileMonitorGnomeVFSPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GNOME_VFS (desktop_agnostic_vfs_trash_gnome_vfs_get_type ())
#define DESKTOP_AGNOSTIC_VFS_TRASH_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GNOME_VFS, DesktopAgnosticVFSTrashGnomeVFS))
#define DESKTOP_AGNOSTIC_VFS_TRASH_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GNOME_VFS, DesktopAgnosticVFSTrashGnomeVFSClass))
#define DESKTOP_AGNOSTIC_VFS_IS_TRASH_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_IS_TRASH_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_TRASH_GNOME_VFS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_GNOME_VFS, DesktopAgnosticVFSTrashGnomeVFSClass))

typedef struct _DesktopAgnosticVFSTrashGnomeVFS DesktopAgnosticVFSTrashGnomeVFS;
typedef struct _DesktopAgnosticVFSTrashGnomeVFSClass DesktopAgnosticVFSTrashGnomeVFSClass;
typedef struct _DesktopAgnosticVFSTrashGnomeVFSPrivate DesktopAgnosticVFSTrashGnomeVFSPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_VOLUME (desktop_agnostic_vfs_trash_volume_get_type ())
#define DESKTOP_AGNOSTIC_VFS_TRASH_VOLUME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_VOLUME, DesktopAgnosticVFSTrashVolume))
#define DESKTOP_AGNOSTIC_VFS_TRASH_VOLUME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_VOLUME, DesktopAgnosticVFSTrashVolumeClass))
#define DESKTOP_AGNOSTIC_VFS_IS_TRASH_VOLUME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_VOLUME))
#define DESKTOP_AGNOSTIC_VFS_IS_TRASH_VOLUME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_VOLUME))
#define DESKTOP_AGNOSTIC_VFS_TRASH_VOLUME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_TRASH_VOLUME, DesktopAgnosticVFSTrashVolumeClass))

typedef struct _DesktopAgnosticVFSTrashVolume DesktopAgnosticVFSTrashVolume;
typedef struct _DesktopAgnosticVFSTrashVolumeClass DesktopAgnosticVFSTrashVolumeClass;

#define DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GNOME_VFS (desktop_agnostic_vfs_volume_gnome_vfs_get_type ())
#define DESKTOP_AGNOSTIC_VFS_VOLUME_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GNOME_VFS, DesktopAgnosticVFSVolumeGnomeVFS))
#define DESKTOP_AGNOSTIC_VFS_VOLUME_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GNOME_VFS, DesktopAgnosticVFSVolumeGnomeVFSClass))
#define DESKTOP_AGNOSTIC_VFS_IS_VOLUME_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_IS_VOLUME_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_VOLUME_GNOME_VFS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_GNOME_VFS, DesktopAgnosticVFSVolumeGnomeVFSClass))

typedef struct _DesktopAgnosticVFSVolumeGnomeVFS DesktopAgnosticVFSVolumeGnomeVFS;
typedef struct _DesktopAgnosticVFSVolumeGnomeVFSClass DesktopAgnosticVFSVolumeGnomeVFSClass;
typedef struct _DesktopAgnosticVFSVolumeGnomeVFSPrivate DesktopAgnosticVFSVolumeGnomeVFSPrivate;

#define DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GNOME_VFS (desktop_agnostic_vfs_volume_monitor_gnome_vfs_get_type ())
#define DESKTOP_AGNOSTIC_VFS_VOLUME_MONITOR_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GNOME_VFS, DesktopAgnosticVFSVolumeMonitorGnomeVFS))
#define DESKTOP_AGNOSTIC_VFS_VOLUME_MONITOR_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GNOME_VFS, DesktopAgnosticVFSVolumeMonitorGnomeVFSClass))
#define DESKTOP_AGNOSTIC_VFS_IS_VOLUME_MONITOR_GNOME_VFS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_IS_VOLUME_MONITOR_GNOME_VFS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GNOME_VFS))
#define DESKTOP_AGNOSTIC_VFS_VOLUME_MONITOR_GNOME_VFS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_VFS_TYPE_VOLUME_MONITOR_GNOME_VFS, DesktopAgnosticVFSVolumeMonitorGnomeVFSClass))

typedef struct _DesktopAgnosticVFSVolumeMonitorGnomeVFS DesktopAgnosticVFSVolumeMonitorGnomeVFS;
typedef struct _DesktopAgnosticVFSVolumeMonitorGnomeVFSClass DesktopAgnosticVFSVolumeMonitorGnomeVFSClass;
typedef struct _DesktopAgnosticVFSVolumeMonitorGnomeVFSPrivate DesktopAgnosticVFSVolumeMonitorGnomeVFSPrivate;

struct _DesktopAgnosticVFSGnomeVFSImplementation {
	GObject parent_instance;
	DesktopAgnosticVFSGnomeVFSImplementationPrivate * priv;
};

struct _DesktopAgnosticVFSGnomeVFSImplementationClass {
	GObjectClass parent_class;
};

struct _DesktopAgnosticVFSFileGnomeVFS {
	DesktopAgnosticVFSFile parent_instance;
	DesktopAgnosticVFSFileGnomeVFSPrivate * priv;
};

struct _DesktopAgnosticVFSFileGnomeVFSClass {
	DesktopAgnosticVFSFileClass parent_class;
};

struct _DesktopAgnosticVFSFileMonitorGnomeVFS {
	GObject parent_instance;
	DesktopAgnosticVFSFileMonitorGnomeVFSPrivate * priv;
};

struct _DesktopAgnosticVFSFileMonitorGnomeVFSClass {
	GObjectClass parent_class;
};

struct _DesktopAgnosticVFSTrashGnomeVFS {
	GObject parent_instance;
	DesktopAgnosticVFSTrashGnomeVFSPrivate * priv;
	GHashTable* trash_dirs;
};

struct _DesktopAgnosticVFSTrashGnomeVFSClass {
	GObjectClass parent_class;
};

struct _DesktopAgnosticVFSVolumeGnomeVFS {
	GObject parent_instance;
	DesktopAgnosticVFSVolumeGnomeVFSPrivate * priv;
};

struct _DesktopAgnosticVFSVolumeGnomeVFSClass {
	GObjectClass parent_class;
};

struct _DesktopAgnosticVFSVolumeMonitorGnomeVFS {
	GObject parent_instance;
	DesktopAgnosticVFSVolumeMonitorGnomeVFSPrivate * priv;
};

struct _DesktopAgnosticVFSVolumeMonitorGnomeVFSClass {
	GObjectClass parent_class;
};


GType desktop_agnostic_vfs_gnome_vfs_implementation_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSGnomeVFSImplementation* desktop_agnostic_vfs_gnome_vfs_implementation_new (void);
DesktopAgnosticVFSGnomeVFSImplementation* desktop_agnostic_vfs_gnome_vfs_implementation_construct (GType object_type);
GType register_plugin (void);
GType desktop_agnostic_vfs_file_gnome_vfs_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSFileGnomeVFS* desktop_agnostic_vfs_file_gnome_vfs_new (void);
DesktopAgnosticVFSFileGnomeVFS* desktop_agnostic_vfs_file_gnome_vfs_construct (GType object_type);
GType desktop_agnostic_vfs_file_monitor_gnome_vfs_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSFileMonitorGnomeVFS* desktop_agnostic_vfs_file_monitor_gnome_vfs_new (DesktopAgnosticVFSFileGnomeVFS* file);
DesktopAgnosticVFSFileMonitorGnomeVFS* desktop_agnostic_vfs_file_monitor_gnome_vfs_construct (GType object_type, DesktopAgnosticVFSFileGnomeVFS* file);
GType desktop_agnostic_vfs_trash_gnome_vfs_get_type (void) G_GNUC_CONST;
GType desktop_agnostic_vfs_trash_volume_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSTrashGnomeVFS* desktop_agnostic_vfs_trash_gnome_vfs_new (void);
DesktopAgnosticVFSTrashGnomeVFS* desktop_agnostic_vfs_trash_gnome_vfs_construct (GType object_type);
GType desktop_agnostic_vfs_volume_gnome_vfs_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSVolumeGnomeVFS* desktop_agnostic_vfs_volume_gnome_vfs_new (void);
DesktopAgnosticVFSVolumeGnomeVFS* desktop_agnostic_vfs_volume_gnome_vfs_construct (GType object_type);
GType desktop_agnostic_vfs_volume_monitor_gnome_vfs_get_type (void) G_GNUC_CONST;
DesktopAgnosticVFSVolumeMonitorGnomeVFS* desktop_agnostic_vfs_volume_monitor_gnome_vfs_new (void);
DesktopAgnosticVFSVolumeMonitorGnomeVFS* desktop_agnostic_vfs_volume_monitor_gnome_vfs_construct (GType object_type);


G_END_DECLS

#endif
