/* da-fdo-gnome.h generated by valac 0.10.4, the Vala compiler, do not modify */


#ifndef __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_FDO_GNOME_H__
#define __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_FDO_GNOME_H__

#include <glib.h>
#include <glib-object.h>
#include <libdesktop-agnostic/fdo.h>

G_BEGIN_DECLS


#define DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GNOME (desktop_agnostic_fdo_desktop_entry_gnome_get_type ())
#define DESKTOP_AGNOSTIC_FDO_DESKTOP_ENTRY_GNOME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GNOME, DesktopAgnosticFDODesktopEntryGNOME))
#define DESKTOP_AGNOSTIC_FDO_DESKTOP_ENTRY_GNOME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GNOME, DesktopAgnosticFDODesktopEntryGNOMEClass))
#define DESKTOP_AGNOSTIC_FDO_IS_DESKTOP_ENTRY_GNOME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GNOME))
#define DESKTOP_AGNOSTIC_FDO_IS_DESKTOP_ENTRY_GNOME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GNOME))
#define DESKTOP_AGNOSTIC_FDO_DESKTOP_ENTRY_GNOME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GNOME, DesktopAgnosticFDODesktopEntryGNOMEClass))

typedef struct _DesktopAgnosticFDODesktopEntryGNOME DesktopAgnosticFDODesktopEntryGNOME;
typedef struct _DesktopAgnosticFDODesktopEntryGNOMEClass DesktopAgnosticFDODesktopEntryGNOMEClass;
typedef struct _DesktopAgnosticFDODesktopEntryGNOMEPrivate DesktopAgnosticFDODesktopEntryGNOMEPrivate;

struct _DesktopAgnosticFDODesktopEntryGNOME {
	GObject parent_instance;
	DesktopAgnosticFDODesktopEntryGNOMEPrivate * priv;
};

struct _DesktopAgnosticFDODesktopEntryGNOMEClass {
	GObjectClass parent_class;
};


GType desktop_agnostic_fdo_desktop_entry_gnome_get_type (void) G_GNUC_CONST;
DesktopAgnosticFDODesktopEntryGNOME* desktop_agnostic_fdo_desktop_entry_gnome_new (void);
DesktopAgnosticFDODesktopEntryGNOME* desktop_agnostic_fdo_desktop_entry_gnome_construct (GType object_type);
GType register_plugin (void);


G_END_DECLS

#endif
