/* da-fdo-glib.h generated by valac 0.10.4, the Vala compiler, do not modify */


#ifndef __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_FDO_GLIB_H__
#define __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_FDO_GLIB_H__

#include <glib.h>
#include <glib-object.h>
#include <libdesktop-agnostic/fdo.h>

G_BEGIN_DECLS


#define DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GLIB (desktop_agnostic_fdo_desktop_entry_glib_get_type ())
#define DESKTOP_AGNOSTIC_FDO_DESKTOP_ENTRY_GLIB(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GLIB, DesktopAgnosticFDODesktopEntryGLib))
#define DESKTOP_AGNOSTIC_FDO_DESKTOP_ENTRY_GLIB_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GLIB, DesktopAgnosticFDODesktopEntryGLibClass))
#define DESKTOP_AGNOSTIC_FDO_IS_DESKTOP_ENTRY_GLIB(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GLIB))
#define DESKTOP_AGNOSTIC_FDO_IS_DESKTOP_ENTRY_GLIB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GLIB))
#define DESKTOP_AGNOSTIC_FDO_DESKTOP_ENTRY_GLIB_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_FDO_TYPE_DESKTOP_ENTRY_GLIB, DesktopAgnosticFDODesktopEntryGLibClass))

typedef struct _DesktopAgnosticFDODesktopEntryGLib DesktopAgnosticFDODesktopEntryGLib;
typedef struct _DesktopAgnosticFDODesktopEntryGLibClass DesktopAgnosticFDODesktopEntryGLibClass;
typedef struct _DesktopAgnosticFDODesktopEntryGLibPrivate DesktopAgnosticFDODesktopEntryGLibPrivate;

struct _DesktopAgnosticFDODesktopEntryGLib {
	GObject parent_instance;
	DesktopAgnosticFDODesktopEntryGLibPrivate * priv;
};

struct _DesktopAgnosticFDODesktopEntryGLibClass {
	GObjectClass parent_class;
};


GType desktop_agnostic_fdo_desktop_entry_glib_get_type (void) G_GNUC_CONST;
DesktopAgnosticFDODesktopEntryGLib* desktop_agnostic_fdo_desktop_entry_glib_new (void);
DesktopAgnosticFDODesktopEntryGLib* desktop_agnostic_fdo_desktop_entry_glib_construct (GType object_type);
GType register_plugin (void);


G_END_DECLS

#endif
