/* da-cfg-keyfile.h generated by valac 0.10.4, the Vala compiler, do not modify */


#ifndef __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_CFG_KEYFILE_H__
#define __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_CFG_KEYFILE_H__

#include <glib.h>
#include <libdesktop-agnostic/config.h>

G_BEGIN_DECLS


#define DESKTOP_AGNOSTIC_CONFIG_TYPE_GKEY_FILE (desktop_agnostic_config_gkey_file_get_type ())
#define DESKTOP_AGNOSTIC_CONFIG_GKEY_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_CONFIG_TYPE_GKEY_FILE, DesktopAgnosticConfigGKeyFile))
#define DESKTOP_AGNOSTIC_CONFIG_GKEY_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_CONFIG_TYPE_GKEY_FILE, DesktopAgnosticConfigGKeyFileClass))
#define DESKTOP_AGNOSTIC_CONFIG_IS_GKEY_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_CONFIG_TYPE_GKEY_FILE))
#define DESKTOP_AGNOSTIC_CONFIG_IS_GKEY_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_CONFIG_TYPE_GKEY_FILE))
#define DESKTOP_AGNOSTIC_CONFIG_GKEY_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_CONFIG_TYPE_GKEY_FILE, DesktopAgnosticConfigGKeyFileClass))

typedef struct _DesktopAgnosticConfigGKeyFile DesktopAgnosticConfigGKeyFile;
typedef struct _DesktopAgnosticConfigGKeyFileClass DesktopAgnosticConfigGKeyFileClass;
typedef struct _DesktopAgnosticConfigGKeyFilePrivate DesktopAgnosticConfigGKeyFilePrivate;

struct _DesktopAgnosticConfigGKeyFile {
	DesktopAgnosticConfigBackend parent_instance;
	DesktopAgnosticConfigGKeyFilePrivate * priv;
};

struct _DesktopAgnosticConfigGKeyFileClass {
	DesktopAgnosticConfigBackendClass parent_class;
};


GType desktop_agnostic_config_gkey_file_get_type (void) G_GNUC_CONST;
DesktopAgnosticConfigGKeyFile* desktop_agnostic_config_gkey_file_new (void);
DesktopAgnosticConfigGKeyFile* desktop_agnostic_config_gkey_file_construct (GType object_type);
GType register_plugin (void);


G_END_DECLS

#endif
