/* da-cfg-gconf.h generated by valac 0.10.4, the Vala compiler, do not modify */


#ifndef __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_CFG_GCONF_H__
#define __DEFAULT_LIBDESKTOP_AGNOSTIC_DA_CFG_GCONF_H__

#include <glib.h>
#include <libdesktop-agnostic/config.h>

G_BEGIN_DECLS


#define DESKTOP_AGNOSTIC_CONFIG_TYPE_GCONF_BACKEND (desktop_agnostic_config_gconf_backend_get_type ())
#define DESKTOP_AGNOSTIC_CONFIG_GCONF_BACKEND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESKTOP_AGNOSTIC_CONFIG_TYPE_GCONF_BACKEND, DesktopAgnosticConfigGConfBackend))
#define DESKTOP_AGNOSTIC_CONFIG_GCONF_BACKEND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), DESKTOP_AGNOSTIC_CONFIG_TYPE_GCONF_BACKEND, DesktopAgnosticConfigGConfBackendClass))
#define DESKTOP_AGNOSTIC_CONFIG_IS_GCONF_BACKEND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), DESKTOP_AGNOSTIC_CONFIG_TYPE_GCONF_BACKEND))
#define DESKTOP_AGNOSTIC_CONFIG_IS_GCONF_BACKEND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), DESKTOP_AGNOSTIC_CONFIG_TYPE_GCONF_BACKEND))
#define DESKTOP_AGNOSTIC_CONFIG_GCONF_BACKEND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), DESKTOP_AGNOSTIC_CONFIG_TYPE_GCONF_BACKEND, DesktopAgnosticConfigGConfBackendClass))

typedef struct _DesktopAgnosticConfigGConfBackend DesktopAgnosticConfigGConfBackend;
typedef struct _DesktopAgnosticConfigGConfBackendClass DesktopAgnosticConfigGConfBackendClass;
typedef struct _DesktopAgnosticConfigGConfBackendPrivate DesktopAgnosticConfigGConfBackendPrivate;

struct _DesktopAgnosticConfigGConfBackend {
	DesktopAgnosticConfigBackend parent_instance;
	DesktopAgnosticConfigGConfBackendPrivate * priv;
};

struct _DesktopAgnosticConfigGConfBackendClass {
	DesktopAgnosticConfigBackendClass parent_class;
};


GType desktop_agnostic_config_gconf_backend_get_type (void) G_GNUC_CONST;
DesktopAgnosticConfigGConfBackend* desktop_agnostic_config_gconf_backend_new (void);
DesktopAgnosticConfigGConfBackend* desktop_agnostic_config_gconf_backend_construct (GType object_type);
GType register_plugin (void);


G_END_DECLS

#endif
